from __future__ import annotations

from typing import Any

from packaging import version as packaging_version

from checkov.common.bridgecrew.severities import BcSeverities, Severities
from checkov.common.models.enums import CheckResult, ScanDataFormat
from checkov.common.sca.output import create_report_cve_record, create_report_license_record
from checkov.runner_filter import RunnerFilter
from checkov.sca_package.output import (
    calculate_lowest_compliant_version,
    create_cli_cves_table,
    create_cli_license_violations_table,
    create_cli_output,
    CveCount,
)


def get_vulnerabilities_details() -> list[dict[str, Any]]:
    return [
        {
            "id": "CVE-2019-19844",
            "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
            "cvss": 9.8,
            "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
            "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
            "severity": "critical",
            "packageName": "django",
            "packageVersion": "1.2",
            "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
            "riskFactors": [
                "Attack complexity: low",
                "Attack vector: network",
                "Critical severity",
                "Has fix",
            ],
            "impactedVersions": ["<1.11.27"],
            "publishedDate": "2019-12-18T20:15:00+01:00",
            "discoveredDate": "2019-12-18T19:15:00Z",
            "fixDate": "2019-12-18T20:15:00+01:00",
        },
        {
            "id": "CVE-2016-6186",
            "status": "fixed in 1.9.8, 1.8.14",
            "cvss": 6.1,
            "vector": "CVSS:3.0/AV:N/AC:L/PR:N/UI:R/S:C/C:L/I:L/A:N",
            "description": "Cross-site scripting (XSS) vulnerability in the dismissChangeRelatedObjectPopup function ...",
            "severity": "medium",
            "packageName": "django",
            "packageVersion": "1.2",
            "link": "https://nvd.nist.gov/vuln/detail/CVE-2016-6186",
            "riskFactors": [
                "Attack complexity: low",
                "Attack vector: network",
                "Exploit exists",
                "Has fix",
                "Medium severity",
            ],
            "impactedVersions": ["<=1.8.13"],
            "publishedDate": "2016-08-05T17:59:00+02:00",
            "discoveredDate": "2016-08-05T15:59:00Z",
            "fixDate": "2016-08-05T17:59:00+02:00",
        },
    ]


def test_create_report_cve_record():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "critical",
        "packageName": "django",
        "packageVersion": "1.12",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }

    # when
    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses='OSI_BDS',
        package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
    )

    # then
    assert record.bc_check_id == "BC_CVE_2019_19844"
    assert record.check_id == "CKV_CVE_2019_19844"
    assert record.check_class == check_class
    assert record.check_name == "SCA package scan"
    assert record.check_result == {"result": CheckResult.FAILED}
    assert record.code_block == [(0, "django: 1.12")]
    assert (
        record.description
        == "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ..."
    )
    assert record.file_abs_path == file_abs_path
    assert record.file_line_range == [0, 0]
    assert record.file_path == f"/{rootless_file_path}"
    assert record.repo_file_path == file_abs_path
    assert record.resource == "requirements.txt.django"
    assert record.severity == Severities[BcSeverities.CRITICAL]
    assert record.short_description == "CVE-2019-19844 - django: 1.12"
    assert record.vulnerability_details["status"] == "fixed in 3.0.1, 2.2.9, 1.11.27"
    assert record.vulnerability_details["lowest_fixed_version"] == "2.2.9"
    assert record.vulnerability_details["fixed_versions"] == [
        packaging_version.parse("3.0.1"),
        packaging_version.parse("2.2.9"),
    ]
    assert record.vulnerability_details["licenses"] == 'OSI_BDS'


def test_create_report_cve_record_results_from_platform():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    vulnerability_details = {
                "severity": "CRITICAL",
                "riskFactors": "{\"Critical severity\":{},\"Attack vector: network\":{},\"Has fix\":{},\"Attack complexity: low\":{}}",
                "id": "CVE-2019-19844",
                "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
                "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. A suitably crafted email address (that is equal to an existing user\\'s email address after case transformation of Unicode characters) would allow an attacker to be sent a password reset token for the matched user account. (One mitigation in the new releases is to send password reset tokens only to the registered user email address.)",
                "packageVersion": "1.2",
                "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                "packageName": "django",
                "publishedDate": "2019-12-18T19:15:00Z",
                "cvss": 9.8,
                "status": "OPEN",
                "cveStatus": "1.11.27",
                "fileMetadataId": "d9f631f2-86b3-4d47-9b23-a2529c255392",
                "ViolationResource": {
                    "scannerType": "Twistcli",
                    "customerName": "ipeleg",
                    "status": "OPEN",
                    "firstDetectionDate": None,
                    "updatedDate": "2022-08-23T09:37:27.207Z",
                    "resourceId": "/packages/requirements.txt",
                    "violationId": "BC_VUL_2",
                    "sourceId": "itai1357/terragoat1",
                    "ticket": None,
                    "metadataFixId": None,
                    "originalResourceDefinition": None,
                    "fixedResourceDefinition": None,
                    "errorLine": None,
                    "resourcePlanId": None,
                    "errorLines": None,
                    "variableCode": None,
                    "variableFixCode": None,
                    "resourceFixCode": None,
                    "gitBlameMetadataId": None
                },
                "isRootPackage": None,
                "packageId": "49d27c4c-68cc-4eeb-ab98-d40a11334fdf",
                "causePackageId": "49d27c4c-68cc-4eeb-ab98-d40a11334fdf",
            }

    # when
    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses='OSI_BDS',
        scan_data_format=ScanDataFormat.PLATFORM,
        package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
    )

    # then

    # in the case of scan_data_format=ScanDataFormat.FROM_PLATFORM we just have to make sure that 'status' and
    # 'fix_version' are as expected, as the rest are the same as in default flow
    # (can_data_format=ScanDataFormat.FROM_TWISTCLI)
    assert "lowest_fixed_version" not in record.vulnerability_details
    assert "fixed_versions" not in record.vulnerability_details
    assert record.vulnerability_details["fix_version"] == '1.11.27'


def test_create_report_cve_record_moderate_severity():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "moderate",
        "packageName": "django",
        "packageVersion": "1.2",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }

    # when
    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses='OSI_BDS',
        package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
    )

    # then
    assert record.severity == Severities[BcSeverities.MEDIUM]


def test_create_report_cve_record_severity_filter():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "moderate",
        "packageName": "django",
        "packageVersion": "1.2",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }

    # when
    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        runner_filter=RunnerFilter(checks=['HIGH']),
        licenses='OSI_BDS',
        package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
    )

    # then
    assert record.bc_check_id == "BC_CVE_2019_19844"
    assert record.check_id == "CKV_CVE_2019_19844"
    assert record.check_class == check_class
    assert record.check_name == "SCA package scan"
    assert record.check_result == {"result": CheckResult.SKIPPED, 'suppress_comment': 'Filtered by severity'}
    assert record.code_block == [(0, "django: 1.2")]
    assert (
        record.description
        == "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ..."
    )
    assert record.file_abs_path == file_abs_path
    assert record.file_line_range == [0, 0]
    assert record.file_path == f"/{rootless_file_path}"
    assert record.repo_file_path == file_abs_path
    assert record.resource == "requirements.txt.django"
    assert record.severity == Severities[BcSeverities.MEDIUM]
    assert record.short_description == "CVE-2019-19844 - django: 1.2"
    assert record.vulnerability_details["lowest_fixed_version"] == "1.11.27"
    assert record.vulnerability_details["fixed_versions"] == [
        packaging_version.parse("3.0.1"),
        packaging_version.parse("2.2.9"),
        packaging_version.parse("1.11.27"),
    ]
    assert record.vulnerability_details["licenses"] == 'OSI_BDS'


def test_create_report_cve_record_package_filter():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "critical",
        "packageName": "django",
        "packageVersion": "1.2",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }

    # when
    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        runner_filter=RunnerFilter(skip_cve_package=['django', 'requests']),
        licenses='OSI_BDS',
        package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
    )

    # then
    assert record.bc_check_id == "BC_CVE_2019_19844"
    assert record.check_id == "CKV_CVE_2019_19844"
    assert record.check_class == check_class
    assert record.check_name == "SCA package scan"
    assert record.check_result == {"result": CheckResult.SKIPPED, "suppress_comment": "Filtered by package 'django'"}
    assert record.code_block == [(0, "django: 1.2")]
    assert (
        record.description
        == "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ..."
    )
    assert record.file_abs_path == file_abs_path
    assert record.file_line_range == [0, 0]
    assert record.file_path == f"/{rootless_file_path}"
    assert record.repo_file_path == file_abs_path
    assert record.resource == "requirements.txt.django"
    assert record.severity == Severities[BcSeverities.CRITICAL]
    assert record.short_description == "CVE-2019-19844 - django: 1.2"
    assert record.vulnerability_details["lowest_fixed_version"] == "1.11.27"
    assert record.vulnerability_details["fixed_versions"] == [
        packaging_version.parse("3.0.1"),
        packaging_version.parse("2.2.9"),
        packaging_version.parse("1.11.27"),
    ]
    assert record.vulnerability_details["licenses"] == 'OSI_BDS'


def test_calculate_lowest_compliant_version():
    # given
    package_versions_list = [
        ["3.0.1", "2.2.9", "1.11.27", "1.9.8"],
        ["1.9.8", "1.8.14"],
        ["1.9.10", "1.8.15"],
        ["3.2.4", "3.1.12", "2.2.24"],
    ]

    fix_versions_lists = [
        [packaging_version.parse(version) for version in package_versions] for package_versions in package_versions_list
    ]

    # when
    compliant_version = calculate_lowest_compliant_version(fix_versions_lists)

    # then
    assert compliant_version == "2.2.24"


def test_create_cli_cves_table(mocker):
    mocker.patch("checkov.common.output.report.CHECKOV_RUN_SCA_PACKAGE_SCAN_V2", return_value=True)
    # given
    file_path = "/path/to/requirements.txt"
    cve_count = CveCount(total=6, critical=0, high=3, medium=2, low=0, skipped=1, has_fix=5, to_fix=5)
    package_details_map = {
        "django": {
            "cves": [
                {"id": "CVE-2016-7401", "severity": "high", "fixed_version": "1.8.15"},
                {"id": "CVE-2016-6186", "severity": "medium", "fixed_version": "1.8.14"},
                {"id": "CVE-2021-33203", "severity": "medium", "fixed_version": "2.2.24"},
            ],
            "current_version": "1.2",
            "compliant_version": "2.2.24",
        },
        "flask": {
            "cves": [
                {"id": "CVE-2019-1010083", "severity": "high", "fixed_version": "1.0"},
                {"id": "CVE-2018-1000656", "severity": "high", "fixed_version": "0.12.3"},
            ],
            "current_version": "0.6",
            "compliant_version": "1.0",
        },
    }

    # when
    table = create_cli_cves_table(
        file_path=file_path,
        cve_count=cve_count,
        package_details_map=package_details_map,
    )

    # then
    assert table == "".join(
        [
            "\t/path/to/requirements.txt - CVEs Summary:\n",
            "\t┌────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┐\n",
            "\t│ Total CVEs: 6      │ critical: 0        │ high: 3            │ medium: 2          │ low: 0             │ skipped: 1         │\n",
            "\t├────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┤\n",
            "\t│ To fix 5/5 CVEs, go to https://www.bridgecrew.cloud/                                                                        │\n",
            "\t├────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┤\n",
            "\t│ Package            │ CVE ID             │ Severity           │ Current version    │ Fixed version      │ Compliant version  │\n",
            "\t├────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┤\n",
            "\t│ django             │ CVE-2016-7401      │ high               │ 1.2                │ 1.8.15             │ 2.2.24             │\n",
            "\t│                    │ CVE-2016-6186      │ medium             │                    │ 1.8.14             │                    │\n",
            "\t│                    │ CVE-2021-33203     │ medium             │                    │ 2.2.24             │                    │\n",
            "\t├────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┤\n",
            "\t│ flask              │ CVE-2019-1010083   │ high               │ 0.6                │ 1.0                │ 1.0                │\n",
            "\t│                    │ CVE-2018-1000656   │ high               │                    │ 0.12.3             │                    │\n",
            "\t└────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┘\n",
        ]
    )


def test_create_cli_license_violations_table(mocker):
    mocker.patch("checkov.common.output.report.CHECKOV_RUN_SCA_PACKAGE_SCAN_V2", return_value=True)

    # given
    file_path = "/requirements.txt"

    package_licenses_details_map = {
        "django": [
            {
                "package_name": "django",
                "package_version": "1.2",
                "license": "DUMMY_LICENSE",
                "status": "OPEN",
                "policy": "BC_LIC_1"
            },
            {
                "package_name": "django",
                "package_version": "1.2",
                "license": "DUMMY_LICENSE2",
                "status": "OPEN",
                "policy": "BC_LIC_1"
            },
        ],
        "flask": [
            {
                "package_name": "flask",
                "package_version": "0.6",
                "license": "DUMMY_LICENSE3",
                "status": "OPEN",
                "policy": "BC_LIC_1"
            },
        ]
    }

    # when
    table = create_cli_license_violations_table(
        file_path=file_path,
        package_licenses_details_map=package_licenses_details_map
    )

    # then
    assert table == "".join(
        [
            "\t/requirements.txt - Licenses Statuses:\n",
            "\t┌────────────────────────┬────────────────────────┬────────────────────────┬────────────────────────┬─────────────────────────┐\n",
            "\t│ Package name           │ Package version        │ Policy ID              │ License                │ Status                  │\n",
            "\t├────────────────────────┼────────────────────────┼────────────────────────┼────────────────────────┼─────────────────────────┤\n",
            "\t│ django                 │ 1.2                    │ BC_LIC_1               │ DUMMY_LICENSE          │ OPEN                    │\n",
            "\t│                        │                        │ BC_LIC_1               │ DUMMY_LICENSE2         │ OPEN                    │\n",
            "\t├────────────────────────┼────────────────────────┼────────────────────────┼────────────────────────┼─────────────────────────┤\n",
            "\t│ flask                  │ 0.6                    │ BC_LIC_1               │ DUMMY_LICENSE3         │ OPEN                    │\n",
            "\t└────────────────────────┴────────────────────────┴────────────────────────┴────────────────────────┴─────────────────────────┘\n",
        ]
    )


def test_create_cli_cves_table_with_no_found_vulnerabilities(mocker):
    mocker.patch("checkov.common.output.report.CHECKOV_RUN_SCA_PACKAGE_SCAN_V2", return_value=True)

    # given
    file_path = "/path/to/requirements.txt"
    cve_count = CveCount(total=2, critical=0, high=0, medium=0, low=0, skipped=2, has_fix=0, to_fix=0)
    package_details_map = {}

    # when
    table = create_cli_cves_table(
        file_path=file_path,
        cve_count=cve_count,
        package_details_map=package_details_map,
    )

    # then
    assert table == "".join(
        [
            "\t/path/to/requirements.txt - CVEs Summary:\n",
            "\t┌────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┐\n",
            "\t│ Total CVEs: 2      │ critical: 0        │ high: 0            │ medium: 0          │ low: 0             │ skipped: 2         │\n",
            "\t├────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┤\n",
            "\t│ To fix 0/0 CVEs, go to https://www.bridgecrew.cloud/                                                                        │\n",
            "\t└─────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘\n",
        ]
    )


def test_create_cli_output(mocker):
    mocker.patch("checkov.common.output.report.CHECKOV_RUN_SCA_PACKAGE_SCAN_V2", return_value=True)
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    license_statuses = [
        {
            "package_name": "django",
            "package_version": "1.2",
            "license": "DUMMY_LICENSE",
            "status": "OPEN",
            "policy": "BC_LIC_1"
        },
        {
            "package_name": "flask",
            "package_version": "0.6",
            "license": "DUMMY_OTHER_LICENSE",  # not a real license. it is just for test a package with 2 licenses
            "status": "OPEN",
            "policy": "BC_LIC_1"
        }
    ]
    # when
    cves_records = [
        create_report_cve_record(
            rootless_file_path=rootless_file_path,
            file_abs_path=file_abs_path,
            check_class=check_class,
            vulnerability_details=details,
            licenses='Unknown',
            package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
        )
        for details in get_vulnerabilities_details()
    ]
    license_records = [
            create_report_license_record(
                rootless_file_path=rootless_file_path,
                file_abs_path=file_abs_path,
                check_class=check_class,
                licenses_status=license_status,
                package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
            )
            for license_status in license_statuses
        ]
    cli_output = create_cli_output(True, cves_records + license_records)

    # then
    assert cli_output == "".join(
        [
            "\t/requirements.txt - CVEs Summary:\n",
            "\t┌────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┐\n",
            "\t│ Total CVEs: 2      │ critical: 1        │ high: 0            │ medium: 1          │ low: 0             │ skipped: 0         │\n",
            "\t├────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┤\n",
            "\t│ To fix 2/2 CVEs, go to https://www.bridgecrew.cloud/                                                                        │\n",
            "\t├────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┤\n",
            "\t│ Package            │ CVE ID             │ Severity           │ Current version    │ Fixed version      │ Compliant version  │\n",
            "\t├────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┤\n",
            "\t│ django             │ CVE-2019-19844     │ critical           │ 1.2                │ 1.11.27            │ 1.11.27            │\n",
            "\t│                    │ CVE-2016-6186      │ medium             │                    │ 1.8.14             │                    │\n",
            "\t└────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┘\n",
            "\n",
            "\t/requirements.txt - Licenses Statuses:\n",
            "\t┌────────────────────────┬────────────────────────┬────────────────────────┬────────────────────────┬─────────────────────────┐\n",
            "\t│ Package name           │ Package version        │ Policy ID              │ License                │ Status                  │\n",
            "\t├────────────────────────┼────────────────────────┼────────────────────────┼────────────────────────┼─────────────────────────┤\n",
            "\t│ django                 │ 1.2                    │ BC_LIC_1               │ DUMMY_LICENSE          │ FAILED                  │\n",
            "\t├────────────────────────┼────────────────────────┼────────────────────────┼────────────────────────┼─────────────────────────┤\n",
            "\t│ flask                  │ 0.6                    │ BC_LIC_1               │ DUMMY_OTHER_LICENSE    │ FAILED                  │\n",
            "\t└────────────────────────┴────────────────────────┴────────────────────────┴────────────────────────┴─────────────────────────┘\n",
        ]
    )


def test_create_cli_output_without_license_records(mocker):
    mocker.patch("checkov.common.output.report.CHECKOV_RUN_SCA_PACKAGE_SCAN_V2", return_value=True)
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    # when
    cves_records = [
        create_report_cve_record(
            rootless_file_path=rootless_file_path,
            file_abs_path=file_abs_path,
            check_class=check_class,
            vulnerability_details=details,
            licenses='Unknown',
            package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
        )
        for details in get_vulnerabilities_details()
    ]
    cli_output = create_cli_output(True, cves_records)

    # then
    assert cli_output == "".join(
        [
            "\t/requirements.txt - CVEs Summary:\n",
            "\t┌────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┐\n",
            "\t│ Total CVEs: 2      │ critical: 1        │ high: 0            │ medium: 1          │ low: 0             │ skipped: 0         │\n",
            "\t├────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┤\n",
            "\t│ To fix 2/2 CVEs, go to https://www.bridgecrew.cloud/                                                                        │\n",
            "\t├────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┬────────────────────┤\n",
            "\t│ Package            │ CVE ID             │ Severity           │ Current version    │ Fixed version      │ Compliant version  │\n",
            "\t├────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┼────────────────────┤\n",
            "\t│ django             │ CVE-2019-19844     │ critical           │ 1.2                │ 1.11.27            │ 1.11.27            │\n",
            "\t│                    │ CVE-2016-6186      │ medium             │                    │ 1.8.14             │                    │\n",
            "\t└────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┴────────────────────┘\n",
        ]
    )


def test_create_cli_output_without_cve_records(mocker):
    mocker.patch("checkov.common.output.report.CHECKOV_RUN_SCA_PACKAGE_SCAN_V2", return_value=True)
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package.scanner.Scanner"
    license_statuses = [
        {
            "package_name": "django",
            "package_version": "1.2",
            "license": "DUMMY_LICENSE",
            "status": "OPEN",
            "policy": "BC_LIC_1"
        },
        {
            "package_name": "flask",
            "package_version": "0.6",
            "license": "DUMMY_OTHER_LICENSE",  # not a real license. it is just for test a package with 2 licenses
            "status": "OPEN",
            "policy": "BC_LIC_1"
        }
    ]
    # when
    license_records = [
            create_report_license_record(
                rootless_file_path=rootless_file_path,
                file_abs_path=file_abs_path,
                check_class=check_class,
                licenses_status=license_status,
                package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
            )
            for license_status in license_statuses
        ]
    cli_output = create_cli_output(True, license_records)

    # then
    assert cli_output == "".join(
        [
            "\t/requirements.txt - Licenses Statuses:\n",
            "\t┌────────────────────────┬────────────────────────┬────────────────────────┬────────────────────────┬─────────────────────────┐\n",
            "\t│ Package name           │ Package version        │ Policy ID              │ License                │ Status                  │\n",
            "\t├────────────────────────┼────────────────────────┼────────────────────────┼────────────────────────┼─────────────────────────┤\n",
            "\t│ django                 │ 1.2                    │ BC_LIC_1               │ DUMMY_LICENSE          │ FAILED                  │\n",
            "\t├────────────────────────┼────────────────────────┼────────────────────────┼────────────────────────┼─────────────────────────┤\n",
            "\t│ flask                  │ 0.6                    │ BC_LIC_1               │ DUMMY_OTHER_LICENSE    │ FAILED                  │\n",
            "\t└────────────────────────┴────────────────────────┴────────────────────────┴────────────────────────┴─────────────────────────┘\n",
        ]
    )



from pathlib import Path

from checkov.arm.graph_builder.graph_components.block_types import BlockType
from checkov.arm.graph_builder.local_graph import ArmLocalGraph
from checkov.arm.utils import get_files_definitions

EXAMPLES_DIR = Path(__file__).parent.parent / "examples"


def test_build_graph():
    # given
    test_file = EXAMPLES_DIR / "container_instance.json"
    definitions, _ = get_files_definitions([str(test_file)])

    local_graph = ArmLocalGraph(definitions=definitions)

    # when
    local_graph.build_graph(render_variables=False)

    # then
    assert len(local_graph.vertices) == 15
    assert len(local_graph.edges) == 0

    assert len(local_graph.vertices_by_block_type[BlockType.PARAMETER]) == 11
    assert len(local_graph.vertices_by_block_type[BlockType.RESOURCE]) == 4



from checkov.common.images.image_referencer import Image
from checkov.azure_pipelines.image_referencer.provider import AzurePipelinesProvider
from checkov.azure_pipelines.image_referencer.manager import AzurePipelinesImageReferencerManager


def test_provider_extract_images_from_workflow():
    file_path = 'tests/azure_pipelines/image_referencer/resources/azure-pipelines.yaml'
    workflow_config = {
    "trigger":
    [
        "master"
    ],
    "resources":
    {
        "repositories":
        [
            {
                "repository": "AzureDevOps",
                "type": "git",
                "endpoint": "AzureDevOps",
                "name": "AzureDevOps/AzureDevOps",
                "__startline__": 6,
                "__endline__": 11
            }
        ],
        "__startline__": 5,
        "__endline__": 11
    },
    "stages":
    [
        {
            "stage": "Example",
            "jobs":
            [
                {
                    "job": "FailNoTag",
                    "displayName": "FailNoTagDisplayName",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 17,
                        "__endline__": 18
                    },
                    "steps":
                    [
                        {
                            "task": "Docker@2",
                            "inputs":
                            {
                                "container": "postgres:14.2",
                                "__startline__": 21,
                                "__endline__": 23
                            },
                            "__startline__": 19,
                            "__endline__": 23
                        }
                    ],
                    "__startline__": 14,
                    "__endline__": 23
                },
                {
                    "job": "PassDigest",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 25,
                        "__endline__": 27
                    },
                    "container": "nginx:1.17",
                    "steps":
                    [
                        {
                            "script": "printenv",
                            "__startline__": 30,
                            "__endline__": 32
                        }
                    ],
                    "__startline__": 23,
                    "__endline__": 32
                }
            ],
            "__startline__": 12,
            "__endline__": 32
        }
    ],
    "jobs":
    [
        {
            "job": "MyJob",
            "container": "ruby:2.6",
            "pool":
            {
                "vmImage": "windows-latest",
                "__startline__": 36,
                "__endline__": 37
            },
            "steps":
            [
                {
                    "script": "echo \"Running in a container based on myorg/mycontainer:1.0\"",
                    "__startline__": 38,
                    "__endline__": 39
                }
            ],
            "__startline__": 33,
            "__endline__": 39
        },
        {
            "job": "MyJob2",
            "container":
            {
                "image": "ruby:2.6",
                "__startline__": 41,
                "__endline__": 42
            },
            "pool":
            {
                "vmImage": "ubuntu-latest",
                "__startline__": 43,
                "__endline__": 44
            },
            "steps":
            [
                {
                    "script": "echo \"Running in a container based on ruby:2.6\"",
                    "__startline__": 45,
                    "__endline__": 45
                }
            ],
            "__startline__": 39,
            "__endline__": 45
        }
    ],
    "__startline__": 1,
    "__endline__": 45
}

    azure_pipelines_provider = AzurePipelinesProvider(workflow_config=workflow_config, file_path=file_path)
    images = azure_pipelines_provider.extract_images_from_workflow()

    assert set(images) == {
        Image(
            end_line=23,
            start_line=21,
            file_path=file_path,
            name='postgres:14.2',
            related_resource_id='stages[0](Example).jobs[0](FailNoTagDisplayName).steps[0].inputs'
        ),
        Image(
            end_line=32,
            start_line=23,
            file_path=file_path,
            name='nginx:1.17',
            related_resource_id='stages[0](Example).jobs[1](PassDigest)'
        ),
        Image(
            end_line=39,
            start_line=33,
            file_path=file_path,
            name='ruby:2.6',
            related_resource_id='jobs[0](MyJob)'
        ),
        Image(
            end_line=45,
            start_line=39,
            file_path=file_path,
            name='ruby:2.6',
            related_resource_id='jobs[1](MyJob2)'
        )
    }

def test_provider_extract_images_from_workflow_no_images():
    file_path = 'tests/azure_pipelines/image_referencer/resources/azure-pipelines.yaml'
    workflow_config = {
    "trigger":
    [
        "master"
    ],
    "resources":
    {
        "repositories":
        [
            {
                "repository": "AzureDevOps",
                "type": "git",
                "endpoint": "AzureDevOps",
                "name": "AzureDevOps/AzureDevOps",
                "__startline__": 6,
                "__endline__": 11
            }
        ],
        "__startline__": 5,
        "__endline__": 11
    },
    "stages":
    [
        {
            "stage": "Example",
            "jobs":
            [
                {
                    "job": "FailNoTag",
                    "displayName": "FailNoTagDisplayName",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 17,
                        "__endline__": 19
                    },
                    "__startline__": 14,
                    "__endline__": 19
                },
                {
                    "job": "PassDigest",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 21,
                        "__endline__": 23
                    },
                    "steps":
                    [
                        {
                            "script": "printenv",
                            "__startline__": 24,
                            "__endline__": 26
                        }
                    ],
                    "__startline__": 19,
                    "__endline__": 26
                }
            ],
            "__startline__": 12,
            "__endline__": 26
        }
    ],
    "jobs":
    [
        {
            "job": "MyJob",
            "pool":
            {
                "vmImage": "windows-latest",
                "__startline__": 29,
                "__endline__": 30
            },
            "steps":
            [
                {
                    "script": "echo \"Running in a container based on myorg/mycontainer:1.0\"",
                    "__startline__": 31,
                    "__endline__": 31
                }
            ],
            "__startline__": 27,
            "__endline__": 31
        }
    ],
    "__startline__": 1,
    "__endline__": 31
}

    azure_pipelines_provider = AzurePipelinesProvider(workflow_config=workflow_config, file_path=file_path)
    images = azure_pipelines_provider.extract_images_from_workflow()

    assert not images

def test_manager_extract_images_from_workflow():
    file_path = 'tests/azure_pipelines/image_referencer/resources/azure-pipelines.yaml'
    workflow_config = {
    "trigger":
    [
        "master"
    ],
    "resources":
    {
        "repositories":
        [
            {
                "repository": "AzureDevOps",
                "type": "git",
                "endpoint": "AzureDevOps",
                "name": "AzureDevOps/AzureDevOps",
                "__startline__": 6,
                "__endline__": 11
            }
        ],
        "__startline__": 5,
        "__endline__": 11
    },
    "stages":
    [
        {
            "stage": "Example",
            "jobs":
            [
                {
                    "job": "FailNoTag",
                    "displayName": "FailNoTagDisplayName",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 17,
                        "__endline__": 18
                    },
                    "steps":
                    [
                        {
                            "task": "Docker@2",
                            "inputs":
                            {
                                "container": "postgres:14.2",
                                "__startline__": 21,
                                "__endline__": 23
                            },
                            "__startline__": 19,
                            "__endline__": 23
                        }
                    ],
                    "__startline__": 14,
                    "__endline__": 23
                },
                {
                    "job": "PassDigest",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 25,
                        "__endline__": 27
                    },
                    "container": "nginx:1.17",
                    "steps":
                    [
                        {
                            "script": "printenv",
                            "__startline__": 30,
                            "__endline__": 32
                        }
                    ],
                    "__startline__": 23,
                    "__endline__": 32
                }
            ],
            "__startline__": 12,
            "__endline__": 32
        }
    ],
    "jobs":
    [
        {
            "job": "MyJob",
            "container": "ruby:2.6",
            "pool":
            {
                "vmImage": "windows-latest",
                "__startline__": 36,
                "__endline__": 37
            },
            "steps":
            [
                {
                    "script": "echo \"Running in a container based on myorg/mycontainer:1.0\"",
                    "__startline__": 38,
                    "__endline__": 39
                }
            ],
            "__startline__": 33,
            "__endline__": 39
        },
        {
            "job": "MyJob2",
            "container":
            {
                "image": "ruby:2.6",
                "__startline__": 41,
                "__endline__": 42
            },
            "pool":
            {
                "vmImage": "ubuntu-latest",
                "__startline__": 43,
                "__endline__": 44
            },
            "steps":
            [
                {
                    "script": "echo \"Running in a container based on ruby:2.6\"",
                    "__startline__": 45,
                    "__endline__": 45
                }
            ],
            "__startline__": 39,
            "__endline__": 45
        }
    ],
    "__startline__": 1,
    "__endline__": 45
}

    manager = AzurePipelinesImageReferencerManager(workflow_config=workflow_config, file_path=file_path)
    images = manager.extract_images_from_workflow()

    assert set(images) == {
        Image(
            end_line=23,
            start_line=21,
            file_path=file_path,
            name='postgres:14.2',
            related_resource_id='stages[0](Example).jobs[0](FailNoTagDisplayName).steps[0].inputs'
        ),
        Image(
            end_line=32,
            start_line=23,
            file_path=file_path,
            name='nginx:1.17',
            related_resource_id='stages[0](Example).jobs[1](PassDigest)'
        ),
        Image(
            end_line=39,
            start_line=33,
            file_path=file_path,
            name='ruby:2.6',
            related_resource_id='jobs[0](MyJob)'
        ),
        Image(
            end_line=45,
            start_line=39,
            file_path=file_path,
            name='ruby:2.6',
            related_resource_id='jobs[1](MyJob2)'
        )
    }

def test_manager_extract_images_from_workflow_no_images():
    file_path = 'tests/azure_pipelines/image_referencer/resources/azure-pipelines.yaml'
    workflow_config = {
    "trigger":
    [
        "master"
    ],
    "resources":
    {
        "repositories":
        [
            {
                "repository": "AzureDevOps",
                "type": "git",
                "endpoint": "AzureDevOps",
                "name": "AzureDevOps/AzureDevOps",
                "__startline__": 6,
                "__endline__": 11
            }
        ],
        "__startline__": 5,
        "__endline__": 11
    },
    "stages":
    [
        {
            "stage": "Example",
            "jobs":
            [
                {
                    "job": "FailNoTag",
                    "displayName": "FailNoTagDisplayName",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 17,
                        "__endline__": 19
                    },
                    "__startline__": 14,
                    "__endline__": 19
                },
                {
                    "job": "PassDigest",
                    "pool":
                    {
                        "vmImage": "ubuntu-18.04",
                        "__startline__": 21,
                        "__endline__": 23
                    },
                    "steps":
                    [
                        {
                            "script": "printenv",
                            "__startline__": 24,
                            "__endline__": 26
                        }
                    ],
                    "__startline__": 19,
                    "__endline__": 26
                }
            ],
            "__startline__": 12,
            "__endline__": 26
        }
    ],
    "jobs":
    [
        {
            "job": "MyJob",
            "pool":
            {
                "vmImage": "windows-latest",
                "__startline__": 29,
                "__endline__": 30
            },
            "steps":
            [
                {
                    "script": "echo \"Running in a container based on myorg/mycontainer:1.0\"",
                    "__startline__": 31,
                    "__endline__": 31
                }
            ],
            "__startline__": 27,
            "__endline__": 31
        }
    ],
    "__startline__": 1,
    "__endline__": 31
}

    manager = AzurePipelinesImageReferencerManager(workflow_config=workflow_config, file_path=file_path)
    images = manager.extract_images_from_workflow()

    assert not images



from checkov.circleci_pipelines.image_referencer.manager import CircleCIImageReferencerManager


def test_extract_images_from_workflow(circle_ci_filepath_workflow_with_images_config,
                                      circle_ci_image1, circle_ci_image2):
    file_path, config = circle_ci_filepath_workflow_with_images_config

    manager = CircleCIImageReferencerManager(file_path=file_path, workflow_config=config)
    images = manager.extract_images_from_workflow()

    assert set(images) == {circle_ci_image1, circle_ci_image2}


def test_extract_images_from_workflow_no_images(circle_ci_filepath_workflow_no_images_config):
    file_path, config = circle_ci_filepath_workflow_no_images_config

    manager = CircleCIImageReferencerManager(file_path=file_path, workflow_config=config)
    images = manager.extract_images_from_workflow()

    assert not images

def test_extract_images_from_workflow_nested(circle_ci_filepath_workflow_no_images_config):
    file_path = '/tmp/test_path'
    config = {
        'workspace_root': '/go/src/github.com/gruntwork-io/terragrunt',
        'defaults': {
            'working_directory': '/go/src/github.com/gruntwork-io/terragrunt', 'docker': [
                {'image': '087285199408.dkr.ecr.us-east-1.amazonaws.com/circle-ci-test-image-base:go1.11',
                    '__startline__': 6, '__endline__': 8}], '__startline__': 3, '__endline__': 8
        },
        'jobs': {
            'install_dependencies': {
                'working_directory': '/go/src/github.com/gruntwork-io/terragrunt', 'docker': [
                {'image': '087285199408.dkr.ecr.us-east-1.amazonaws.com/circle-ci-test-image-base:go1.11',
                 '__startline__': 6, '__endline__': 8}], 'steps': ['checkout'], '__startline__': 11, '__endline__': 32
            }
        }, '__startline__': 1, '__endline__': 143
    }

    manager = CircleCIImageReferencerManager(file_path=file_path, workflow_config=config)
    images = manager.extract_images_from_workflow()

    assert not images



import os
import unittest
import warnings
from unittest import mock

from checkov.terraform import checks
from .test_yaml_policies import load_yaml_data, get_policy_results


class TestYamlConnectedNodes(unittest.TestCase):
    def setUp(self) -> None:
        warnings.filterwarnings("ignore", category=ResourceWarning)
        warnings.filterwarnings("ignore", category=DeprecationWarning)

    def test_S3BucketEncryption_connected_node(self):
        report = self.get_report("S3BucketEncryption")
        assert report.failed_checks[0].connected_node is None
        assert report.failed_checks[1].connected_node['file_path'] == '/main.tf'
        assert report.failed_checks[1].connected_node['resource'] == 'aws_s3_bucket_server_side_encryption_configuration.bad_sse_1'
        assert report.failed_checks[1].connected_node['file_line_range'] == [163, 172]
        assert report.failed_checks[2].connected_node['file_path'] == '/main.tf'
        assert report.failed_checks[2].connected_node['resource'] == 'aws_s3_bucket_server_side_encryption_configuration.bad_sse_2'
        assert report.failed_checks[2].connected_node['file_line_range'] == [174, 182]
        assert report.failed_checks[3].connected_node is None
        assert report.failed_checks[4].connected_node['file_path'] == '/main.tf'
        assert report.failed_checks[4].connected_node['resource'] == 'aws_s3_bucket_server_side_encryption_configuration.bad_sse_3'
        assert report.failed_checks[4].connected_node['file_line_range'] == [184, 195]

        assert report.passed_checks[0].connected_node is None
        assert report.passed_checks[1].connected_node is None
        assert report.passed_checks[2].connected_node is None
        assert report.passed_checks[3].connected_node['file_path'] == '/main.tf'
        assert report.passed_checks[3].connected_node['resource'] == 'aws_s3_bucket_server_side_encryption_configuration.good_sse_1'
        assert report.passed_checks[3].connected_node['file_line_range'] == [117, 126]
        assert report.passed_checks[4].connected_node['file_path'] == '/main.tf'
        assert report.passed_checks[4].connected_node['resource'] == 'aws_s3_bucket_server_side_encryption_configuration.good_sse_2'
        assert report.passed_checks[4].connected_node['file_line_range'] == [128, 137]
        assert report.passed_checks[5].connected_node['file_path'] == '/main.tf'
        assert report.passed_checks[5].connected_node['resource'] == 'aws_s3_bucket_server_side_encryption_configuration.good_sse_3'
        assert report.passed_checks[5].connected_node['file_line_range'] == [139, 150]

    def test_S3BucketLogging_connected_node(self):
        report = self.get_report("S3BucketLogging")
        assert report.failed_checks[0].connected_node is None

        assert report.passed_checks[0].connected_node is None
        assert report.passed_checks[1].connected_node['file_path'] == '/main.tf'
        assert report.passed_checks[1].connected_node['resource'] == 'aws_s3_bucket_logging.example'
        assert report.passed_checks[1].connected_node['file_line_range'] == [14, 19]

    def get_report(self, dir_name):
        dir_path = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                f"resources/{dir_name}")
        assert os.path.exists(dir_path)
        policy_dir_path = os.path.dirname(checks.__file__)
        assert os.path.exists(policy_dir_path)
        for root, _, f_names in os.walk(policy_dir_path):
            for f_name in f_names:
                if f_name != f"{dir_name}.yaml":
                    continue
                policy = load_yaml_data(f_name, root)
                assert policy is not None
                with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': 'NETWORKX'}):
                    # connected nodes don't exist in igraph, because they are not needed
                    return get_policy_results(dir_path, policy)



import os
import re
import unittest

from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.util.data_structures_utils import merge_dicts
from checkov.common.util.http_utils import normalize_prisma_url, normalize_bc_url


class TestUtils(unittest.TestCase):

    def test_merge_dicts(self):
        dict1 = {'a': '1', 'b': '2'}
        dict2 = {'a': '4', 'c': '3'}
        dict3 = {'x': 'x', 'y': 'y', 'a': 'q'}

        res = merge_dicts(dict1, dict2)
        self.assertEqual(len(res), 3)
        self.assertEqual(res['a'], '4')
        self.assertEqual(res['b'], '2')
        self.assertEqual(res['c'], '3')

        res = merge_dicts(dict1, dict2, dict3)
        self.assertEqual(len(res), 5)
        self.assertEqual(res['a'], 'q')
        self.assertEqual(res['b'], '2')
        self.assertEqual(res['c'], '3')
        self.assertEqual(res['x'], 'x')
        self.assertEqual(res['y'], 'y')

        res = merge_dicts(dict1, None)
        self.assertEqual(len(res), 2)
        self.assertEqual(res['a'], '1')
        self.assertEqual(res['b'], '2')

        res = merge_dicts(dict1, 7)
        self.assertEqual(len(res), 2)
        self.assertEqual(res['a'], '1')
        self.assertEqual(res['b'], '2')

    def test_normalize_prisma_url(self):
        self.assertEqual('https://api0.prismacloud.io', normalize_prisma_url('https://api0.prismacloud.io'))
        self.assertEqual('https://api0.prismacloud.io', normalize_prisma_url('https://app0.prismacloud.io'))
        self.assertEqual('https://api0.prismacloud.io', normalize_prisma_url('http://api0.prismacloud.io'))
        self.assertEqual('https://api0.prismacloud.io', normalize_prisma_url('https://api0.prismacloud.io/'))
        self.assertEqual('https://api0.prismacloud.io', normalize_prisma_url(' https://api0.prismacloud.io'))
        self.assertIsNone(normalize_prisma_url(''))
        self.assertIsNone(normalize_prisma_url(None))

    def test_normalize_bc_url(self):
        self.assertEqual('https://www.bridgecrew.cloud', normalize_bc_url('https://www.bridgecrew.cloud'))
        self.assertEqual('https://www.bridgecrew.cloud', normalize_bc_url('http://www.bridgecrew.cloud'))
        self.assertEqual('https://www.bridgecrew.cloud', normalize_bc_url('https://www.bridgecrew.cloud/'))
        self.assertEqual('https://www.bridgecrew.cloud', normalize_bc_url(' https://www.bridgecrew.cloud'))
        self.assertIsNone(normalize_bc_url(''))
        self.assertIsNone(normalize_bc_url(None))

    def test_skip_comment_regex(self):
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'checkov:skip=CKV_AWS_145: ADD REASON'))
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'checkov:skip=CKV_AWS_145:ADD REASON'))
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'checkov:skip=CKV_AWS_145'))
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'bridgecrew:skip=CKV_AWS_145:ADD REASON'))
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'bridgecrew:skip=CKV_AWS_145'))
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'bridgecrew:skip=BC_AWS_GENERAL_123'))
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'bridgecrew:skip=bcorg_AWS_1234567'))
        self.assertIsNotNone(re.search(COMMENT_REGEX, 'checkov:skip=bcorg_AWS_1234567'))



if __name__ == '__main__':
    unittest.main()
